local http = require("socket.http")
local ltn12 = require("ltn12")

function handle_redirect(r)
    local url = "http://127.0.0.1:" .. os.getenv("WEBFILE_PORT") .. r.uri -- 控制端URL
    if r.args then
        url = url .. "?" .. r.args
    end
    local response_body = {}
    local request_headers = {}
    local result_code = apache2.DECLINED  -- 默认返回值

    if r.headers_in["cookie"] then
        request_headers["cookie"] = r.headers_in["cookie"]
    end

    if r.headers_in["authorization"] then
        request_headers["authorization"] = r.headers_in["authorization"]
    end

    local status, response_or_error = pcall(function()
        local body, code, headers = http.request{
            url = url,
            method = "GET",
            headers = request_headers,
            sink = ltn12.sink.table(response_body)
        }

        if not headers then
            r:err("Failed to make HTTP request or no response headers.")
            return apache2.HTTP_SERVICE_UNAVAILABLE -- 确保返回状态码
        end

        if code and code ~= 200 then
            return code -- 确保返回状态码
        end

        if headers["set-cookie"] then
            local token = headers["set-cookie"]:match("([^;]+)")
            r.subprocess_env["WEBFILE_TOKEN"] = token or headers["set-cookie"]
            r.headers_out["Set-Cookie"] = headers["set-cookie"]
        end

        if headers["x-webfile-token"] then
            if r.headers_in["cookie"] then
                if string.find(r.headers_in["cookie"], "webfile_token=", 1, true) then
                    -- 把原来的token替换成新的
                    local pattern = "(webfile_token=)[^;]+"
                    r.headers_in["cookie"] = string.gsub(r.headers_in["cookie"], pattern, headers["x-webfile-token"])
                else
                    -- 原来没有token，添加进去
                    r.headers_in["cookie"] = r.headers_in["cookie"] .. ';' .. headers["x-webfile-token"]
                end
            else
                -- 原来没有cookie，添加进去
                r.headers_in["cookie"] = headers["x-webfile-token"]
            end
        end

        -- 检查是否有重定向
        -- http.request会自动重定向，在这里不行，所以让上游返回值200
        if code and code == 200 and headers.location then
            r.subprocess_env["PROXY_TARGET"] = headers.location
            return apache2.OK -- 确保返回状态码
        else
            return apache2.HTTP_BAD_REQUEST -- 确保返回状态码
        end
    end)

    if not status then
        r:err("Error handling redirect: " .. tostring(response_or_error))
        result_code = apache2.HTTP_INTERNAL_SERVER_ERROR
    else
        result_code = response_or_error
    end

    -- 如果pcall成功，返回内部函数的结果，否则返回错误状态码
    return result_code
end
